@@ -3,9 +3,12 @@
  * @brief Header file for JSON class
  * @author Jonathan Thomas <jonathan@openshot.org>
  *
- * @section LICENSE
+ * @ref License
+ */
+
+/* LICENSE
  *
- * Copyright (c) 2008-2014 OpenShot Studios, LLC
+ * Copyright (c) 2008-2019 OpenShot Studios, LLC
  * <http://www.openshotstudios.com/>. This file is part of
  * OpenShot Library (libopenshot), an open-source project dedicated to
  * delivering high quality video editing and animation solutions to the
@@ -28,6 +31,12 @@
 #ifndef OPENSHOT_JSON_H
 #define OPENSHOT_JSON_H
 
-#include "../thirdparty/jsoncpp/include/json/json.h"
+#include <string>
+#include "json/json.h"
+#include "Exceptions.h"
+
+namespace openshot {
+    const Json::Value stringToJson(const std::string value);
+}
 
 #endif
